(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[37],{1811:function(e,t,r){"use strict";r.r(t),r.d(t,"default",(function(){return l}));r(0);var i=r(34),s=r(62),n=r(1);var l=function(e){var t=e.disabled,r=e.element,l=e.widgetMgr,a={width:e.width},c="primary"===r.type?i.b.PRIMARY:i.b.SECONDARY;return Object(n.jsx)("div",{className:"row-widget stButton",style:a,children:Object(n.jsx)(i.a,{help:r.help,children:Object(n.jsx)(i.d,{kind:c,size:i.c.SMALL,disabled:t,onClick:function(){return l.setTriggerValue(r,{fromUi:!0})},children:Object(n.jsx)(s.a,{source:r.label,allowHTML:!1,isButton:!0})})})})}}}]);